"use strict";
module.exports = class SearchEngine {
    type;
    separator;
    encodeComponent;
    force;
    engine;
    urls;
    tag;
    output;
    constructor(type, engineOrUrls, outputOrTag = false, separator = "+", encodeComponent = false, force = false) {
        this.type = type;
        this.separator = separator;
        this.encodeComponent = encodeComponent;
        this.force = force;
        const web = type === "api" || type === "browser";
        ({
            engine: this.engine = null,
            urls: this.urls = null,
            tag: this.tag = null,
            output: this.output,
        } = web
            ? {
                urls: typeof engineOrUrls === "string" ? [engineOrUrls] : engineOrUrls,
                tag: outputOrTag,
                output: type === "api" ? true : null,
            }
            : {
                engine: engineOrUrls,
                output: outputOrTag || null,
            });
    }
    resolve(key, terms, question) {
        function encode(urls, terms, tag, separator, encodeComponent, force) {
            const encodedQuery = terms
                .map(term => term
                .split("+")
                .map(c => encodeComponent ? encodeURI(c) : encodeURIComponent(c))
                .join("%2B"))
                .join(separator), U = urls.map(url => url.replace(tag, encodedQuery));
            return !force
                ? U
                : U.map(url => `data:text/html,<meta name="color-scheme" content="dark light" />
        <meta http-equiv="Refresh" content="0; url=${url}" />`);
        }
        const { type, output, engine, urls, tag, separator, encodeComponent, force, } = this;
        return {
            type,
            output,
            engine: engine === "" ? key : engine,
            question,
            urls: urls === null
                ? urls
                : encode(urls, terms, tag, separator, encodeComponent, force),
        };
    }
}
