"use strict";
module.exports = function (string, engines, alias, SELECTOR, OPERATORS, MATH, TRANSLATE, FALLBACK) {
    function tokenize(input, FALLBACK) {
        const tokens = input.split(" ").filter((token) => token !== ""), spaces = input.length - input.trimStart().length;
        if (spaces > 0)
            tokens.unshift(FALLBACK.at(Math.min(spaces, FALLBACK.length) - 1));
        if (tokens.length < 1)
            throw new RangeError(`Query has no tokens\n[${input}]`);
        return tokens;
    }
    function operate(tokens, OPERATORS, MATH) {
        function unroll(head) {
            const operation = (/(?<key>^:{0}[a-zA-Z]+)(?<operand>:{0}(?:(?:\d)|(?:-\d))(?:[-a-zA-Z\d]*$))/u).exec(head)
                ?.groups;
            return typeof operation === "undefined"
                ? [head]
                : [
                    operation["key"],
                    operation["operand"],
                ];
        }
        function numeric(char, operators = "") {
            return char >= "0" && char <= "9" || operators.includes(char);
        }
        const [head, ...rest] = tokens;
        return numeric(head[0], OPERATORS)
            || head.length > 1
                && head.startsWith(".")
                && numeric(head[1])
            ? [MATH, ...tokens]
            : [...unroll(head), ...rest];
    }
    function select(tokens, SELECTOR, TRANSLATE) {
        const [head, ...rest] = tokens, { selector, index } = (([iSelector, iDot]) => iDot < 0 || iSelector >= 0 && iSelector < iDot
            ? { selector: SELECTOR, index: iSelector }
            : { selector: ".", index: iDot })([SELECTOR, "."].map(s => head.indexOf(s)));
        if (index < 0)
            return tokens;
        else {
            const [pre, ...selected] = head.split(selector), { key = pre, selection = selected.join(selector), tail = rest, } = pre === ""
                ? { key: TRANSLATE }
                : selected.length < 2 && selected[0] === ""
                    ? {
                        selection: rest[0] ?? "",
                        tail: rest.slice(1),
                    }
                    : {};
            return [
                key,
                `${SELECTOR}${selection}`,
                ...tail,
            ];
        }
    }
    if (`${SELECTOR}${OPERATORS}`.includes("."))
        throw new TypeError("Bad selector/operator");
    const [_K, ..._terms] = select(operate(tokenize(string, FALLBACK), OPERATORS, MATH), SELECTOR, TRANSLATE), _key = _K.toLowerCase(), { key = _key, terms = [..._terms], } = _key in engines
        ? {}
        : _key in alias
            ? { key: alias[_key] }
            : {
                key: FALLBACK.at(-1),
                terms: [_K, ..._terms],
            }, question = terms.join(" "), recomposed = `${key} ${question}`;
    return {
        key,
        terms,
        question,
        recomposed,
    };
}
