"use strict";
module.exports = class File {
    mutable;
    static manager = FileManager.iCloud();
    path;
    parent;
    exists;
    constructor(filetype, mutable, { name, folder = "" }) {
        this.mutable = mutable;
        const root = File.manager.bookmarkExists(filetype)
            ? File.manager.bookmarkedPath(filetype)
            : mutable
                ? `${File.manager.libraryDirectory()}/${filetype}`
                : null, subpath = `${folder}/${name}`
            .split("/")
            .filter(node => node !== "");
        if (root === null)
            throw new ReferenceError(`No bookmark for immutable filetype ${filetype}`);
        else if (subpath.length < 1)
            throw new RangeError("Empty subpath");
        this.path = `${root}/${subpath.join("/")}`;
        this.parent = `${root}/${subpath.slice(0, -1).join("/")}`;
        this.exists = File.manager.fileExists(this.path);
    }
    read(fail = false) {
        if (!this.exists) {
            if (fail)
                throw new ReferenceError("File not found", { cause: this.path });
            return "";
        }
        return File.manager.readString(this.path);
    }
    readful() {
        const read = this.read(true);
        if (read === "")
            throw new ReferenceError("Empty file", { cause: this.path });
        return read;
    }
    write(data = null, overwrite = false) {
        const { path } = this;
        if (!this.mutable)
            throw new TypeError("Cannot write readonly file", { cause: path });
        else if (data === null)
            throw new TypeError("Null write data");
        else if (File.manager.isDirectory(path))
            throw new ReferenceError("Write target is folder", { cause: path });
        else if (this.exists) {
            if (overwrite === false)
                throw new ReferenceError("Cannot overwrite mutable file with `overwrite:false`", { cause: path });
        }
        else if (!File.manager.isDirectory(this.parent))
            File.manager.createDirectory(this.parent, true);
        File.manager.writeString(path, Array.isArray(data)
            ? `${data.reverse().join("\n")}\n${this.read()}`
            : typeof data === "object"
                ? JSON.stringify(data)
                : typeof overwrite !== "string"
                    ? String(data)
                    : overwrite === "line"
                        ? `${String(data)}\n${this.read()}`
                        : `${this.read()}${String(data)}`);
    }
    delete() {
        const { path } = this;
        if (!this.mutable)
            throw new TypeError("Cannot delete readonly file", { cause: path });
        if (this.exists)
            File.manager.remove(path);
    }
}
