"use strict";
const File = importModule("./file.js");
module.exports = class App {
    app;
    cache = {};
    constructor() {
        const { name = "" } = this.constructor;
        if (name === "")
            throw new EvalError("App has no name");
        this.app = name;
    }
    get setting() {
        return this.config ??= ((config) => {
            if (typeof config !== "object" || config === null)
                throw new TypeError("Setting file is not JSON");
            return config;
        })(JSON.parse(new File("Setting", false, { name: `${this.app}.json` }).readful()) ?? null);
    }
    get input() {
        return this.getInput() ?? this.synthetic ?? undefined;
    }
    get inputful() {
        const { input = null } = this;
        if (input === null)
            throw new TypeError("Null input");
        return input;
    }
    get inputString() {
        const { input = "" } = this;
        if (typeof input !== "string" && typeof input !== "number")
            throw new TypeError("Non-string input", { cause: input });
        return String(input);
    }
    get inputStringful() {
        return App.stringful(this.inputString, "input");
    }
    static stringful(string = "", cause = "") {
        if (string === "")
            throw new TypeError(`Unstringful: ${cause}`);
        return string;
    }
    static stringfuls(array, cause = "") {
        if (array.length === 0 || !array.every((i) => i !== ""))
            throw new TypeError(`Empty or unstringful array: ${cause}`);
        return array;
    }
    static error(app, error) {
        function cast(e) {
            return typeof e === "object" && e !== null && "message" in e
                ? e
                : JSON.stringify(e);
        }
        const errors = [cast(error)];
        while (typeof errors[0] !== "string" && "cause" in errors[0])
            errors.unshift(cast(errors[0].cause));
        const n = new Notification, [title, body] = (([head, ...rest]) => [`${app}: ${head}`, rest.join("\n")])(errors.map(e => typeof e === "string" ? e : e.message));
        n.title = title;
        n.body = body;
        n.sound = "failure";
        n.schedule().catch((e) => console.error(e));
        console.error(`${title}\n${body}`);
        return new Error(title, { cause: body });
    }
    run(synthetic) {
        try {
            typeof synthetic === "undefined" || (this.synthetic = synthetic);
            return this.output(this.runtime());
        }
        catch (error) {
            throw App.error(this.app, error);
        }
        finally {
            Script.complete();
        }
    }
    subsetting(subpath) {
        return ((config) => {
            if (typeof config !== "object" || config === null)
                throw new TypeError("Setting file is not JSON");
            return config;
        })(JSON.parse(new File("Setting", false, { name: `${this.app}/${App.stringful(subpath, "subsetting")}.json` }).readful()));
    }
    read(...[file]) {
        return this.storage(file).read();
    }
    readful(...[file]) {
        return this.storage(file).readful();
    }
    write(...[data, overwrite = true, file,]) {
        this.storage(file).write(data, overwrite);
    }
    delete(...[file]) {
        this.storage(file).delete();
    }
    storage(file = {}) {
        const { app } = this, { name: basename = app, ext = "txt", } = typeof file === "object"
            ? file
            : { name: file };
        if (basename === "" || ext === "")
            throw new TypeError("Empty filename");
        const name = `${basename}.${ext}`;
        return this.cache[name] ??= new File("Storage", true, { name, folder: app });
    }
    config;
    synthetic;
}

